/**************************************************************************
*
*	Graphics.h
*
*	Graphics API class
*
*	Author: Saxon Druce
*
*	Copyright (c) 2004 Block Software Pty Ltd
*
**************************************************************************/

#ifndef _GRAPHICS_H
#define _GRAPHICS_H

/*************************************************************************/
// Include files
/*************************************************************************/

#include "CommsClient.h"
#include "Vec.h"

/*************************************************************************/
// Defines
/*************************************************************************/

/*************************************************************************/
// Public types
/*************************************************************************/

class Graphics_t : private CommsClient_t
{
public:

	// Constructor
	Graphics_t();

	// Destructor
	~Graphics_t();

	// Connect() - Connects to the specified port on the specified host
	// (name or IP address). Throws an exception on failure to connect.
	// void Connect(
	//	const std::string& HostName,
	//	int Port);
	using CommsClient_t::Connect; // pull into public use from the base class

	// NOTE: All of the following SetToXXX methods take an ID parameter.
	// It is up to the client to decide on the IDs it passes. If the ID
	// is the same as an ID used in a previous method call, the old shape 
	// is replaced with the new shape. If the ID has not been used before,
	// a new shape is created.

	// SetToCircle() - Sets the shape of the specified ID to be a circle
	// with the specified parameters.
	void SetToCircle(
		int ID, 
		const Vec_t& Position,	// centre of the circle, in metres
		double Radius,			// in metres
		int LineWidth,			// in pixels
		int Red,				// 0 - 255
		int Green,				// 0 - 255
		int Blue);				// 0 - 255

	// SetToRectangle() - Sets the shape of the specified ID to be a 
	// rectangle with the specified parameters.
	void SetToRectangle(
		int ID, 
		const Vec_t& Position,	// centre of the rectangle, in metres
		double Width,			// in metres
		double Height,			// in metres
		double Orientation,		// angle with positive x axis, in radians, positive clockwise
		int LineWidth,			// in pixels
		int Red,				// 0 - 255
		int Green,				// 0 - 255
		int Blue);				// 0 - 255

	// SetToLine() - Sets the shape of the specified ID to be a line with 
	// the specified parameters.
	void SetToLine(
		int ID, 
		const Vec_t& Start,		// in metres
		const Vec_t& End,		// in metres
		int LineWidth,			// in pixels
		int Red,				// 0 - 255
		int Green,				// 0 - 255
		int Blue);				// 0 - 255

	// Remove() - Removes the shape of the specified ID, or does nothing
	// if no shape of that ID exists.
	void Remove(int ID);
};

/*************************************************************************/

#endif // _GRAPHICS_H
