/**************************************************************************
*
*	Main.cpp
*
*	Main source file for dumb robot sample programme
*
*	Author: Saxon Druce
*
*	Copyright (c) 2004 Block Software Pty Ltd
*
**************************************************************************/

/*************************************************************************/
// Include files
/*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h>
#include <windows.h>

#include "Controller.h"

/*************************************************************************/
// Defines
/*************************************************************************/

/*************************************************************************/
// Local types
/*************************************************************************/

/*************************************************************************/
// Local functions
/*************************************************************************/

/*************************************************************************/
// Functions
/*************************************************************************/

/**************************************************************************
*
*	function:	main()
*
*	desc:		Programme entry point
*
*	notes:		
*
**************************************************************************/

void main(int NumArgs, char** Args)
{
	try
	{
		printf("Dumb Robot Sample\n\n");

		// seed random number generator
		srand((unsigned int)time(NULL));

		// setup default values of parameters
		std::string ServerName = "localhost";

		// expect arguments to be in the following format:
		// /a a_value /b b_value
		// so there should be an even number of them, after the first one
		// which is the exe filename
		if ((NumArgs - 1) % 2 != 0)
			throw std::runtime_error("Expected an even number of arguments");
		
		// now loop over them
		int i;
		for (i = 1; i < NumArgs; i += 2)
		{
			if (!strcmp(Args[i], "/s"))
			{
				// /s = server address (a string)
				ServerName = Args[i + 1];
			}
			else
			{
				throw std::runtime_error(
					std::string() + "Invalid argument " + Args[i]);
			}
		}

		// create a controller
		Controller_t Controller;

		// connect to the server
		Controller.Connect(ServerName, 9386);

		// create a robot
		int RobotID = Controller.CreateRobot();
		printf("Robot ID is %d\n", RobotID);

		// set name and owner
		Controller.SetName("Dumb Robot");
		Controller.SetOwner("Saxon");

		// get initial amount of money
		printf("Robot has $%d to spend on equipment\n", 
			Controller.GetMoney());

		// buy equipment
		Controller.PurchaseEngine(3, 75);
		Controller.PurchaseSensor(10, 60, 60);
		Controller.PurchaseBattery(50);
		Controller.PurchaseTurret(60, 75);
		Controller.PurchaseWeapon(1.3, 4, 2, 4.5);
		Controller.PurchaseSolarPanel(1.5);

		// get amount of money left
		printf("Robot has $%d remaining after purchasing equipment\n", 
			Controller.GetMoney());

		// enter the simulation
		Controller.EnterSimulation();

		// set throttle to full speed ahead
		Controller.SetThrottle(100.0);

		// now loop until a key is pressed
		printf("\nHit a key to stop\n\n");
		while (!_kbhit())
		{
			// set desired orientation to 90 degrees further clockwise
			// than we currently are, which should result in a constant
			// clockwise spin as fast as possible (assuming we aren't
			// able to spin by 90 degrees between iterations of this loop)
			Controller.SetDesiredOrientation(
				Controller.GetOrientation() + M_PI / 2);

			// do the same thing with the sensor orientation, to make it
			// constantly spin (not that we do anything with it!)
			Controller.SetDesiredSensorOrientation(
				Controller.GetSensorOrientation() + M_PI / 2);

			// set the desired turret orientation to a function which will
			// make it move back and forth over its range
			Controller.SetDesiredTurretOrientation(
				30 * pow(sin(GetTickCount() / 500.0), 3) * M_PI / 180.0);

			// fire the weapon if it's fully charged, wherever we happen
			// to be pointing
			if (Controller.GetWeaponCharge() >= 100)
				Controller.FireWeapon();
		}
		// read the pressed key
		_getch();

		printf("\nDone\n\n");
	}
	catch (std::exception& e)
	{
		printf("\nAn exception occurred: %s\n\n", e.what());
	}
}

/*************************************************************************/
